{% extends "modal.html" %}
{% load i18n %}

{% block "dialog-body" %}
  {% url "alert.settings" as alertsettings_url %}
  {% if alert.default_email %}
    <div>{% blocktrans %}You will receive an email notification by default when you subscribe, but these can be switched off in your <a href="{{ alertsettings_url }}">subscription settings</a>.{% endblocktrans %}</div>
  {% else %}
    <div>{% blocktrans %}You will <strong>NOT</strong> receive an email notification when you subscribe, but these can be switched on in your <a href="{{ alertsettings_url }}">subscription settings</a>.{% endblocktrans %}</div>
  {% endif %}

  <p>{{ alert.desc|safe }}</p>
{% endblock %}
 
{% block "form-buttons"  %}
  <a class="btn btn-default" href="{% url "alert.settings" %}">{% trans "Cancel" %}</a>
  <input type="submit" class="btn btn-primary" value="{% trans "Subscribe Me" %}"/>
{% endblock %}
