{% extends "modal.html" %}
{% load i18n %}

{% block "dialog-body" %}
  {% if reply_to %}
    <form>
    {{ reply_to.body }}
    <hr/>
    </form>
  {% endif %}
{% endblock %}

{% block "form-buttons" %}
  <a href="{% url "alerts" %}" class="btn btn-default">{% trans "Cancel" %}</a>
  {% if reply_to %}
    <button type="submit" class="btn btn-primary">{% trans "Send Reply" %}</button>
  {% else %}
    <button type="submit" class="btn btn-primary">{% trans "Send Message" %}</button>
  {% endif %}
{% endblock %}
