{% extends "modal.html" %}
{% load i18n %}

{% block "dialog-body" %}
  <p>{% blocktrans %}If you unsubscribe, you will no longer get notifications about {{ alert }}.{% endblocktrans %}</p>
{% endblock %}

{% block "form-buttons" %}
  <a class="btn btn-default" href="{% url "alert.settings" %}">{% trans "Cancel" %}</a>
  <input type="submit" class="btn btn-danger" value="{% trans "Unsubscribe" %}"/>
{% endblock %}
