{% load i18n %}
<div class="body">
  <div class="text">{{ instance.body|urlize|linebreaks }}</div>
</div>
<div class="footer">
   <div class="status">{% trans "from" %} <a href="{{ instance.sender.get_absolute_url }}" title="{% trans "Visit profile" %}">{{ instance.sender }}</a></div>
   <div class="index">
     {% if instance %}
       <a href="{% url "message.reply" instance.sender.username instance.pk %}">{% trans "Reply" %}</a>
     {% endif %}
   </div>
</div>
